#!/usr/bin/env python3

import os
import shutil

import importlib.resources

from pstats import f8
from dataclasses import dataclass
from typing import Dict


@dataclass
class FunctionProfile:
    ncalls: int
    tottime: float
    percall_tottime: float
    cumtime: float
    percall_cumtime: float
    file_name: str
    line_number: int
    func_name: str


@dataclass
class StatsProfile:
    """Class for keeping track of an item in inventory."""

    total_tt: float
    func_profiles: Dict[str, FunctionProfile]


def get_stats_profile(stats):
    """This method returns an instance of StatsProfile,
    which contains a mapping of function names to instances
    of FunctionProfile. Each FunctionProfile instance holds
    information related to the function's profile such as how
    long the function took to run, how many times it was called, etc...
    """
    func_list = (
        stats.fcn_list[:] if stats.fcn_list else list(stats.stats.keys())
    )
    if not func_list:
        return StatsProfile(0, {})

    total_tt = float(f8(stats.total_tt))
    func_profiles = {}
    stats_profile = StatsProfile(total_tt, func_profiles)

    for func in func_list:
        cc, nc, tt, ct, callers = stats.stats[func]
        file_name, line_number, func_name = func
        ncalls = str(nc) if nc == cc else (str(nc) + "/" + str(cc))
        tottime = float(f8(tt))
        percall_tottime = -1 if nc == 0 else float(f8(tt / nc))
        cumtime = float(f8(ct))
        percall_cumtime = -1 if cc == 0 else float(f8(ct / cc))
        func_profile = FunctionProfile(
            ncalls,
            tottime,  # time spent in this function alone
            percall_tottime,
            cumtime,  # time spent in the function plus all functions that this function called,
            percall_cumtime,
            file_name,
            line_number,
            func_name,
        )
        func_profiles[func_name] = func_profile

    return stats_profile


def write_css_file(public_html_dir, static_css_file):
    '''
    Copy a css file to the  public html dir
    '''
    with importlib.resources.path('rapidpe_rift_pipe.static', static_css_file) as  p:
       css_path = p
    shutil.copy(css_path, os.path.join(public_html_dir, static_css_file))



#!/usr/bin/env python3

__author__ = "Anarya Ray, Caitlin Rose, Vinaya Valsan"


import os

import numpy as np
import matplotlib.pyplot as plt


def compute_source_prob(rate_dict, evidence_dict):
    """
    Calculates multicomponent source probability

    Parameters
    ----------
    rate_dict : dict
        categorywise rate
    evidence_dict: dict
        categorywise evidence calculated from PE samples

    Returns
    -------
    dict
        a dictionary of probability for astrophysical categories
        such that the sum of probailities equal to 1.

    """
    rates_weighted_evidence_dict = {
        category: evidence_dict[category] * rate
        for category, rate in rate_dict.items()
    }
    rates_weighted_evidence_sum = sum(rates_weighted_evidence_dict.values())
    src_prob = {
        k: z / rates_weighted_evidence_sum
        for k, z in rates_weighted_evidence_dict.items()
    }

    return src_prob


class ZeroSrcProbError(ValueError):
    def __init__(
        self,
        message="Unable to reweight src_prob with p_terr as"
        " all component of src_prob are zero",
    ):
        self.message = message
        super().__init__(self.message)


def pastro(src_prob, p_terr):
    """
    Combines given p_terr with astrophysical source probabilities.

    Parameters
    ----------
    src_prob
        dictionary of astrophysical probabilities
    p_terr
        terrestrial probability

    Returns
    -------
    dict
        a dictionary of pastro normalized such that the sum of astrophysical
        probabilities and terrestrial probability equals to 1.

    """
    if sum(src_prob.values()) != 0.0:
        pastros = {k: (1 - p_terr) * src_p for k, src_p in src_prob.items()}
        pastros["Terrestrial"] = p_terr

        return pastros
    else:
        raise ZeroSrcProbError()


def renormalize_pastro_with_pipeline_pterr(rapidpe_pastro, pipeline_pastro):
    """
    Combines pipeline p_terr with astrophysical source probabilities.

    Parameters
    ----------
    rapidpe_pastro
        dictionary of pastros including pterr from rapidpe
    pipeline_pastro
        dictionary of pastros including pterr from pipeline

    Returns
    -------
    dict
        a dictionary of pastro normalized with pipeine pterr such that the
        sum of astrophysical probabilities and terrestrial probability
        equals to 1.
    """
    pipeline_p_terr = pipeline_pastro["Terrestrial"]
    rapidpe_pterr_complement = 1.0 - rapidpe_pastro["Terrestrial"]
    if rapidpe_pastro["Terrestrial"] != 1.0:
        src_prob = {
            category: rapidpe_pastro_value / rapidpe_pterr_complement
            for category, rapidpe_pastro_value in rapidpe_pastro.items()
            if category != "Terrestrial"
        }
    else:
        src_prob = {
            category: 0.0
            for category, rapidpe_pastro_value in rapidpe_pastro.items()
            if category != "Terrestrial"
        }

    pastros = pastro(src_prob, pipeline_p_terr)

    return pastros


def plot_pastro(pastro_dict, save_dir, filename="p_astro.png"):
    """
    adapted from gwcelery.tasks
    """

    def _format_prob(prob):
        if prob >= 1:
            return "100%"
        elif prob <= 0:
            return "0%"
        elif prob > 0.99:
            return ">99%"
        elif prob < 0.01:
            return "<1%"
        else:
            return "{}%".format(int(np.round(100 * prob)))

    fname = os.path.join(save_dir, filename)

    probs = list(pastro_dict.values())
    names = list(pastro_dict.keys())

    with plt.style.context("seaborn-white"):
        fig, ax = plt.subplots(figsize=(2.5, 2))
        ax.barh(names, probs)
        for i, prob in enumerate(probs):
            ax.annotate(
                _format_prob(prob),
                (0, i),
                (4, 0),
                textcoords="offset points",
                ha="left",
                va="center",
            )
        ax.set_xlim(0, 1)
        ax.set_xticks([])
        ax.tick_params(left=False)
        for side in ["top", "bottom", "right"]:
            ax.spines[side].set_visible(False)
        fig.tight_layout()
        fig.savefig(fname)
    return



from rapidpe_rift_pipe import modules

def test_placeholder():
    assert True


def test_convert_list_string_to_dict():
    test_cases = [
        ("[x=y,a=b]", {"x": "y", "a": "b"}),
        ("[x=1,a=b]", {"x": "1", "a": "b"}),
#        ("[]", {}),
    ]

    for list_string, dct in test_cases:
        assert modules.convert_list_string_to_dict(list_string) == dct
